'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _util = require('../util');

var Maybe = function Maybe(v) {
  if (v == null) {
    return new Nothing();
  }
  return new Just(v);
};

Maybe.of = function (v) {
  if (v instanceof Maybe) {
    return v;
  }
  return new Maybe(v);
};

var Just = (0, _util.inherit)(Maybe, function (v) {
  this.value = v;
});

Just.prototype.map = function (f) {
  return new Maybe(f(this.value));
};

Just.prototype.extract = function () {
  return this.value;
};
Just.prototype.ap = function (v) {
  return v.map(this.value);
};
Just.prototype.chain = function (f) {
  return f(this.value);
};

Just.prototype.toString = function () {
  return 'Just:' + this.value;
};

var Nothing = (0, _util.inherit)(Maybe, function () {
  this.value = undefined;
});

Nothing.prototype.map = function () {
  return new Nothing();
};
Nothing.prototype.ap = function () {
  return this;
};
Nothing.prototype.extract = function () {
  return undefined;
};

Nothing.prototype.toString = function () {
  return 'Nothing';
};
Maybe.Just = Just;
Maybe.Nothing = Nothing;
exports['default'] = Maybe;
module.exports = exports['default'];